import React, { useEffect, useState, useRef, useMemo } from 'react';
import axios from 'axios';

interface testABAttributes {
  components: React.ComponentType[]
  id: string
}

const AB_TEST_URL = 'http://localhost:3001/v1/visit/ab/';
const AB_TEST_VIEW_DURATION_URL = 'http://localhost:3001/v1/visit/ab/view/duration';
const AB_TEST_EVENT = 'http://localhost:3001/v1/visit/ab/add/event';
const AB_TEST_CONVERSION = 'http://localhost:3001/v1/visit/ab/update/conversion';

const TestAB = (props: testABAttributes) => {
	console.log('TestAB');

	// Listening to viewport changes
	const refDiv = useRef<HTMLDivElement>(null);
	const EmptyComponent = () => {
		return <>Loading ...</>;
	};
	const isInViewport1 = useIsInViewport(refDiv);
	function useIsInViewport(ref: React.RefObject<HTMLDivElement>) {
		const [isIntersecting, setIsIntersecting] = useState(false);
		const observer = useMemo(
			() =>
				new IntersectionObserver(([entry]) =>
					setIsIntersecting(entry.isIntersecting),
				),
			[],
		);
  
		useEffect(() => {
			// eslint-disable-next-line @typescript-eslint/ban-ts-comment
			// @ts-ignore
			observer.observe(ref.current);
			return () => {
				observer.disconnect();
			};
		}, [ref, observer]);
		return isIntersecting;
	}

	// Update view duration if the component is in viewport
	useEffect(() => {
		const interval = setInterval(() => {
			const isBrowserActive = ((typeof document. hasFocus != 'undefined' ? document. hasFocus() : 1) ? 1 : 0);
			if (isInViewport1 && isBrowserActive) {
				// Update view duration
				axios.get(AB_TEST_VIEW_DURATION_URL, {withCredentials: true});
			}
		}, 1000);
		return () => clearInterval(interval);
	}, [isInViewport1]);

	const [selectedComponent, setSelectedComponent] = useState<null | number>(null);
	useEffect(() => {

		axios.get(`${AB_TEST_URL}${props.id}`, {withCredentials: true})
			.then(function (res) {
				console.log(res.data);
				setSelectedComponent(res.data.component);
			})
			.catch(function (error) {
				console.error(error.response);
				setSelectedComponent(0);
			});

	}, [props.id]);

	// Collection click data
	const clickOnDiv = () => {
		axios.post(AB_TEST_EVENT, {
			event: 'clicked'
		}, {withCredentials: true});
	};

	return <div ref={refDiv} onClick={clickOnDiv}>{selectedComponent !== null ? React.createElement(props.components[selectedComponent]) : <EmptyComponent />}</div>;
};


// Custom event sending
const Event = {
	send: (name: string) => {
		if (name) {
			axios.post(AB_TEST_EVENT, {
				event: name
			}, {withCredentials: true});
		} else {
			console.error('Event name can\'t be empty!');
		}
	}
};

// Update conversion status
const Conversion = {
	update: (status: boolean) => {
		if (typeof status == 'boolean') {
			axios.post(AB_TEST_CONVERSION, {
				status: status
			}, {withCredentials: true});
		} else {
			console.error('Conversion only accept true/false!');
		}
	}
};

export default TestAB;
export {Event, Conversion};
